==> 0.0.2.tar.gz: lib/src/draw.dart <==
part of draw_your_image;

/// A widget representing a canvas for drawing.
class Draw extends StatefulWidget {
  /// [Color] for background of canvas.
  final Color backgroundColor;

  /// [Color] of strokes as an initial configuration.
  final Color strokeColor;

  /// Width of strokes
  final double strokeWidth;

  const Draw({
    Key? key,
    this.backgroundColor = Colors.white,
    this.strokeColor = Colors.black,
    this.strokeWidth = 4,
  }) : super(key: key);

  @override
  _DrawState createState() => _DrawState();
}

class _DrawState extends State<Draw> {
  late Color _backgroundColor;
  late Color _strokeColor;
  late double _strokeWidth;

  // late Size _canvasSize;
  final _strokes = <_Stroke>[];

  @override
  void initState() {
    _backgroundColor = widget.backgroundColor;
    _strokeColor = widget.strokeColor;
    _strokeWidth = widget.strokeWidth;

    super.initState();
  }

  void _start(double startX, double startY) {
    _strokes.add(
      _Stroke(
        color: _strokeColor,
        width: _strokeWidth,
      ),
    );
    _strokes.last.path.moveTo(startX, startY);
  }

  void _add(double x, double y) {
    setState(() {
      _strokes.last.path.lineTo(x, y);
    });
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: double.infinity,
      width: double.infinity,
      child: GestureDetector(
        onPanStart: (details) => _start(
          details.localPosition.dx,
          details.localPosition.dy,
        ),
        onPanUpdate: (details) {
          _add(
            details.localPosition.dx,
            details.localPosition.dy,
          );
        },
        child: LayoutBuilder(
          builder: (context, constraints) {
            // _canvasSize = Size(constraints.maxWidth, constraints.maxHeight);
            return CustomPaint(
              painter: _FreehandPainter(_strokes, _backgroundColor),
            );
          },
        ),
      ),
    );
  }
}

/// Subclass of [CustomPainter] to paint strokes
class _FreehandPainter extends CustomPainter {
  final List<_Stroke> strokes;
  final Color backgroundColor;

  _FreehandPainter(
    this.strokes,
    this.backgroundColor,
  );

  @override
  void paint(Canvas canvas, Size size) {
    canvas.clipRect(Rect.fromLTWH(0, 0, size.width, size.height));
    canvas.drawRect(
      Rect.fromLTWH(0, 0, size.width, size.height),
      Paint()..color = Colors.white,
    );
    for (final stroke in strokes) {
      final paint = Paint()
        ..strokeWidth = stroke.width
        ..color = stroke.color
        ..strokeCap = StrokeCap.round
        ..style = PaintingStyle.stroke;
      canvas.drawPath(stroke.path, paint);
    }
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}

/// Data class representing strokes
class _Stroke {
  final path = Path();
  final Color color;
  final double width;

  _Stroke({
    this.color = Colors.black,
    this.width = 4,
  });
}

==> 0.0.5.tar.gz: lib/src/draw.dart <==
part of draw_your_image;

/// A widget representing a canvas for drawing.
class Draw extends StatefulWidget {
  /// A controller to call drawing actions.
  final DrawController? controller;

  /// [Color] for background of canvas.
  final Color backgroundColor;

  /// [Color] of strokes as an initial configuration.
  final Color strokeColor;

  /// Width of strokes
  final double strokeWidth;

  /// Callback which is called when [Canvas] is converted to image data.
  /// See [DrawController] to check how to convert.
  final ValueChanged<Uint8List>? onConvert;

  const Draw({
    Key? key,
    this.controller,
    this.backgroundColor = Colors.white,
    this.strokeColor = Colors.black,
    this.strokeWidth = 4,
    this.onConvert,
  }) : super(key: key);

  @override
  _DrawState createState() => _DrawState();
}

class _DrawState extends State<Draw> {
  // late Size _canvasSize;
  final _strokes = <_Stroke>[];

  @override
  void initState() {
    widget.controller?._delegate = _DrawControllerDelegate()
      ..onConvertToPng = () {
        // currently do nothing.
      };
    super.initState();
  }

  void _start(double startX, double startY) {
    _strokes.add(
      _Stroke(
        color: widget.strokeColor,
        width: widget.strokeWidth,
      ),
    );
    _strokes.last.path.moveTo(startX, startY);
  }

  void _add(double x, double y) {
    setState(() {
      _strokes.last.path.lineTo(x, y);
    });
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: double.infinity,
      width: double.infinity,
      child: GestureDetector(
        onPanStart: (details) => _start(
          details.localPosition.dx,
          details.localPosition.dy,
        ),
        onPanUpdate: (details) {
          _add(
            details.localPosition.dx,
            details.localPosition.dy,
          );
        },
        child: LayoutBuilder(
          builder: (context, constraints) {
            // _canvasSize = Size(constraints.maxWidth, constraints.maxHeight);
            return CustomPaint(
              painter: _FreehandPainter(_strokes, widget.backgroundColor),
            );
          },
        ),
      ),
    );
  }
}

/// Subclass of [CustomPainter] to paint strokes
class _FreehandPainter extends CustomPainter {
  final List<_Stroke> strokes;
  final Color backgroundColor;

  _FreehandPainter(
    this.strokes,
    this.backgroundColor,
  );

  @override
  void paint(Canvas canvas, Size size) {
    canvas.clipRect(Rect.fromLTWH(0, 0, size.width, size.height));
    canvas.drawRect(
      Rect.fromLTWH(0, 0, size.width, size.height),
      Paint()..color = Colors.white,
    );
    for (final stroke in strokes) {
      final paint = Paint()
        ..strokeWidth = stroke.width
        ..color = stroke.color
        ..strokeCap = StrokeCap.round
        ..style = PaintingStyle.stroke;
      canvas.drawPath(stroke.path, paint);
    }
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}

/// Data class representing strokes
class _Stroke {
  final path = Path();
  final Color color;
  final double width;

  _Stroke({
    this.color = Colors.black,
    this.width = 4,
  });
}
